
#-*- coding: utf-8 -*-
"""
Todo
----
insert module comments in this space.

"""

__title__ = "gitlab-management"
__version__ = "0.1.6rc200620132451"
__doc__ = "https://gitlab.com/nofusscomputing/projects/python-gitlab-management"
__author__ = "No Fuss Computing"
__email__ = "helpdesk@nofusscomputing.com"
__license__ = "GNU LGPLv3"
__copyright__ = "(C) All Rights reserved"
__source__ = "https://gitlab.com/nofusscomputing/projects/python-gitlab-management/-/tree/358f2a25154c73f9c64effacf41add03368a465a"
